/* $Id: rtunes.c,v 1.43 2006/02/09 19:35:26 hacki Exp $ */

/*
 * Copyright (c) 2006 Marcus Glocker <marcus@nazgul.ch>
 * Copyright (c) 2006 Marc Balmer <mbalmer@openbsd.org>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <err.h>
#include <fcntl.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "config.h"
#include "extern.h"
#include "pathnames.h"
#include "proto.h"

/* prototypes */

static __dead void 	usage(void);
void 			sighandler(const int);
extern void		rtunes_init(void);

/* global vars local */

volatile sig_atomic_t	quit = 0;

/* global vars extern */

char		*cfgfile = _PATH_CFGFILE;
char		*host = NULL;
AES_KEY		aeskey;
unsigned char	*aesiv;
int		sfd_rtsp, sfd_stream;
char		localip[16];

/*
 * usage()
 *	print usage message
 * Return:
 *	none
 */

static __dead void
usage(void)
{
	extern char	*__progname;

	fprintf(stderr, "usage: %s [-f configfile] [-h host] <file(s)>\n",
	    __progname);

	exit(1);
}

/*
 * sighandler()
 *	signal handler
 * Return:
 *	none
 */

void
sighandler(const int sig)
{
	switch (sig) {
	case SIGINT:
		/* FALLTHROUGH */
	case SIGTERM:
		quit = 1;
		break;
	default:
		/* ignore */
		break;
	}
}

/* rtunes streams audio files to Apple's AirPort Express device */

int
main(int argc, char *argv[])
{
	int		e, i, r, s, ch, fd, sample;
	char		cwd[1024];
	unsigned char	*buf;
	struct stco	*o;
	struct stsz	*z;
	struct stsc	*c;

	/* get command line options */

	while ((ch = getopt(argc, argv, "f:h:")) != -1) {
		switch (ch) {
		case 'f':
			cfgfile = optarg;
			break;
		case 'h':
			host = optarg;
			break;
		default:
			usage();
			/* NOTREACHED */
		}
	}

	rtunes_init();

	if (host == NULL)
		usage(); /* no host defined */

	if (!(argc - optind))
		usage(); /* no audio files defined */

	/* get local ip address */

	if (sys_getlocalip(localip, sizeof(localip)) == -1)
		errx(1, "Can't get local hosts IP address");

	/* connect airport rtsp port */

	if ((sfd_rtsp = sys_connect(host, AIRPORT)) == -1)
		err(1, "Connect rtsp port");

	/* rtsp connection sequence */

	if ((r = rtsp_connect()) == -1)
		errx(1, "RTSP connection sequence failed");

	/* connect airport audio stream port */

	if ((sfd_stream = sys_connect(host, r)) == -1)
		err(1, "Connect stream port");

	/* change to current directory */

	if (getcwd(cwd, sizeof(cwd)) == NULL)
		err(1, "getcwd()");
	if (chdir(cwd) == -1)
		err(1, "chdir()");

	/* signal handler */

	signal(SIGINT, sighandler);
	signal(SIGTERM, sighandler);

	/* cycle through all audio files */

	for (i = optind; argv[i] != NULL && !quit; i++) {
		/* open file */
		if ((fd = open(argv[i], O_RDONLY)) == -1) {
			warn("open()");
			continue;
		}

		/* check codec */
		if (mp4_check(fd) != 1) {
			fprintf(stderr, "File is not Apple Lossless\n");
			close(fd);
			continue;
		}

		/* demux */

		/* STCO */
		if (!mp4_seek_box(fd, "stco")) {
			fprintf(stderr, "STCO box not found\n");
			close(fd);
			continue;
		}
		if ((o = mp4_parse_stco(fd)) == NULL) {
			fprintf(stderr, "STCO parse error\n");
			close(fd);
			continue;
		}
		/* STSZ */
		if (!mp4_seek_box(fd, "stsz")) {
			fprintf(stderr, "STSZ box not found\n");
			close(fd);
			continue;
		}
		if ((z = mp4_parse_stsz(fd)) == NULL) {
			fprintf(stderr, "STSZ parse error\n");
			free(o);
			close(fd);
			continue;
		}
		/* STSC */
		if (!mp4_seek_box(fd, "stsc")) {
			fprintf(stderr, "STSC box not found\n");
			close(fd);
			continue;
		}
		if ((c = mp4_parse_stsc(fd)) == NULL) {
			fprintf(stderr, "STSC parse error\n");
			free(o);
			free(z);
			close(fd);
			continue;
		}
		/* MDAT */
		if (!mp4_seek_box(fd, "mdat")) {
			fprintf(stderr, "MDAT box not found\n");
			free(o);
			free(z);
			free(c);
			close(fd);
			continue;
		}

		/* seek to first chunk */
		lseek(fd, o->chunk_offset[1], SEEK_SET);

		/* print some info */
		printf("Playing %s\n", argv[i]);

		/* play */
		for (sample = 1; sample <= z->sample_count && !quit; sample++) {
			buf = malloc(z->entry_size[sample] + 32);

			r = sys_read(fd, buf, z->entry_size[sample]);
			if (r < 1) {
				free(buf);
				break;
			}

			e = cipher_aes_sendsample(buf, r);

			s = sys_write(sfd_stream, buf, e);

			free(buf);
		}

		/* cleanup */
		free(o);
		free(z);
		free(c);
		close(fd);

		/* have a break */
		sleep(1);
	}

	/* disconnect airport */
	
	if (rtsp_disconnect() == -1)
		err(1, "Disconnect failed");

	close(sfd_stream);
	close(sfd_rtsp);

	return (0);
}
